from typing import Optional
from fastapi import APIRouter

from fastapi import Depends
from sqlalchemy.orm import Session
from app import get_page, page_help, schemas

import app.crud as crud
from app.utils.send_util import get_job_run_status
from constants.constants import RUN_STATUS
from utils.sx_time import sxtimeit
from utils.sx_web import web_try
from fastapi_pagination import Page, add_pagination, paginate, Params

from app import get_db

router = APIRouter(
    prefix="/jpt/joblog",
    tags=["joblog-日志管理"],
)


@router.post("/")
@web_try()
@sxtimeit
def create_job_log(item: schemas.JobLogCreate, db: Session = Depends(get_db)):
    return crud.create_job_log(db, item)


@router.get("/")
@web_try()
@sxtimeit
def get_job_logs(job_id: Optional[int] = None, params: Params=Depends(get_page), db: Session = Depends(get_db)):
    job_infos = []
    if job_id is None:
        job_infos = crud.get_job_infos(db)
    else:
        job_infos = [crud.get_job_info(db,job_id)]
    id_to_job = {job.id:job for job in job_infos}
    relations = crud.get_af_ids(db, id_to_job.keys(), 'datax')
    af_to_datax = {relation.af_id:relation.se_id for relation in relations}
    af_job_runs = crud.get_airflow_runs_by_af_job_ids(db, af_to_datax.keys())
    af_job_runs.sort(key=lambda x: x.start_time, reverse=True)
    total = len(af_job_runs)
    af_job_runs = af_job_runs[(params['page'] - 1) * params['size']:params['page'] * params['size']]
    res = []
    for af_job_run in af_job_runs:
        task = list(af_job_run.details['tasks'].values())[0] if len(list(af_job_run.details['tasks'].values()))>0 else None
        job_id = af_to_datax[int(af_job_run.job_id)]
        execute_result = None
        if af_job_run.status <= 1:
            run_status = get_job_run_status(af_job_run.id)
            execute_result = run_status['data']['status']
        log = {
            "id": af_job_run.id,
            "job_id": job_id,
            "job_desc": id_to_job[job_id].job_desc,
            "af_job_id": int(af_job_run.job_id),
            "run_id": af_job_run.af_run_id,
            "trigger_time": af_job_run.start_time,
            "trigger_result": 1,
            "execute_time": task['start_time'] if task else 0,
            "execute_result": execute_result if execute_result else af_job_run.status,
            "end_time": task['end_time'] if task else 0,
            "log": task['log'] if task else None
        }
        res.append(log)
    return page_help(res,params['page'],params['size'],total)


@router.get("/getOnce")
@web_try()
@sxtimeit
def get_job_logs_once(run_id: int, db: Session = Depends(get_db)):
    af_job_run = crud.get_airflow_run_once(db, run_id)
    task = list(af_job_run.details['tasks'].values())[0] if len(list(af_job_run.details['tasks'].values()))>0 else None
    log = {
        "id": af_job_run.id,
        "af_job_id": int(af_job_run.job_id),
        "run_id": af_job_run.af_run_id,
        "trigger_time": af_job_run.start_time,
        "trigger_result": 1,
        "execute_time": task['start_time'] if task else 0,
        "execute_result": af_job_run.status,
        "end_time": task['end_time'] if task else 0,
        "log": task['log'] if task else None
    }
    return log


@router.get("/logs_status/{ids}")
@web_try()
@sxtimeit
def get_job_log_status(ids: str):
    run_ids = ids.split(',')
    id_to_status = {}
    for run_id in run_ids:
        res = get_job_run_status(run_id)
        id_to_status.update({run_id:res['data']['status']})
    return id_to_status